/*
*
* Author: Giacomo Lozito <james@develia.org>, (C) 2005-2006
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or (at your
* option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*
*/

#include "cb.h"
#include "ui.h"
#include "cfg.h"
#include "mon.h"


gboolean
gmu_cb_trayapp_bpress ( GtkWidget * tray_evbox , GdkEventButton * event , gpointer data )
{

  switch ( event->button )
  {
    case 1:
    {
      GtkWidget *tray_mmenu = GTK_WIDGET(g_object_get_data( G_OBJECT(tray_evbox) , "main_menu" ));
      gtk_menu_popup( GTK_MENU(tray_mmenu) , NULL , NULL ,
                      NULL , NULL , event->button , event->time );
      break;
    }

    case 3:
    {
      GtkWidget *tray_cmenu = GTK_WIDGET(g_object_get_data( G_OBJECT(tray_evbox) , "config_menu" ));
      gtk_menu_popup( GTK_MENU(tray_cmenu) , NULL , NULL ,
                      NULL , NULL , event->button , event->time );
      break;
    }
  }

  return TRUE;
}


gboolean
gmu_cb_trayapp_destroy ( GtkWidget * trayapp )
{
  /* call again gmu_ui_trayapp() to work around a eggtrayicon/gtk+ bug that gives
     a X Window error when gmutoo is still running and the X server is shut down */
  DEBUGMSG( "gmutoo tray icon was destroyed (X server shutdown?), shutting down properly...\n" );
  gmu_ui_trayapp();
  return TRUE;
}


void
gmu_cb_mmenu_execute_monitored ( GtkWidget * status_label )
{
  gchar *command;
  guint status = GPOINTER_TO_UINT(g_object_get_data( G_OBJECT(status_label) , "status" ));
  switch ( status )
  {
    case GMU_PROCSTATUS_OFF:
      command = g_object_get_data( G_OBJECT(status_label) , "cmdstart" );
      break;

    case GMU_PROCSTATUS_ON:
      command = g_object_get_data( G_OBJECT(status_label) , "cmdstop" );
      break;
  }
  gmu_cb_mmenu_execute_generic( command );
}


void
gmu_cb_mmenu_execute_generic ( gpointer command )
{
  GError *error = NULL;
  gchar **command_argv = NULL;

  if ( !g_shell_parse_argv( command , NULL , &command_argv , &error ) )
  {
    g_warning( "Unable to parse the command '%s' - error message: %s\n" ,
               command , error->message );
    return;
  }
  DEBUGMSG( "Executing command '%s'...\n" , command );
  error = NULL;
  if ( !g_spawn_async( NULL , command_argv , NULL , 0 , NULL , NULL , NULL , &error ) )
  {
    g_warning( "Unable to execute the command '%s' - error message: %s\n" ,
               command , error->message );
  }
  #ifdef DEBUG
  else { DEBUGMSG( "Command '%s' executed!\n" , command ); }
  #endif /* DEBUG */
  g_strfreev( command_argv );
}


void
gmu_cb_cmenu_quit ( GtkWidget * evbox )
{
  GtkWidget *mmenu = GTK_WIDGET(g_object_get_data( G_OBJECT(evbox) , "main_menu" ));
  GtkWidget *cmenu = GTK_WIDGET(g_object_get_data( G_OBJECT(evbox) , "config_menu" ));

  /* save cfg to file */
  gmu_cfg_write_file( config );

  /* destroy main menu and config menu */
  gtk_widget_destroy( mmenu );
  gtk_widget_destroy( cmenu );

  /* close monitor */
  gmu_mon_shutdown();

  gtk_main_quit();
}


void
gmu_cb_cmenu_config ( GtkWidget * evbox )
{
  gmu_ui_configwin( evbox );
}


void
gmu_cb_configwin_about ( GtkWidget * config_window )
{
  gchar *authors[] = { "Giacomo Lozito" , NULL };
  gtk_show_about_dialog( GTK_WINDOW(config_window) ,
                         "name" , "gMUTOO" ,
                         "version" , PACKAGE_VERSION ,
                         "authors" , authors ,
                         "copyright" , "\302\251 2006 Giacomo Lozito\n<james@develia.org>" ,
                         "license" , "This program is free software; you can redistribute it and/or modify\n"
                                     "it under the terms of the GNU General Public License as published by\n"
                                     "the Free Software Foundation; either version 2 of the License, or\n"
                                     "(at your option) any later version.\n\n"
                                     "This program is distributed in the hope that it will be useful,\n"
                                     "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
                                     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
                                     "GNU General Public License for more details.\n\n"
                                     "You should have received a copy of the GNU General Public License\n"
                                     "along with this program; if not, write to the Free Software\n"
                                     "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n" ,
                         NULL );
}


void
gmu_cb_configwin_add ( GtkWidget * config_window )
{
  gmu_ui_configwin_modwin( GTK_WIDGET(config_window) , GMU_CONFIGWIN_ADD );
}


gboolean
gmu_cb_configwin_commit ( GtkTreeModel *model , GtkTreePath *path ,
                          GtkTreeIter *iter , gpointer data )
{
  gchar *c_name, *c_desc, *c_cmdstart, *c_cmdstop, *c_monproc, *c_iconfile;
  gchar *c_entry;
  gboolean c_ifmon, c_ifenable;

  gtk_tree_model_get( model , iter ,
                      GMU_LVCOLUMN_IFENABLE , &c_ifenable ,
                      GMU_LVCOLUMN_NAME , &c_name ,
                      GMU_LVCOLUMN_DESC , &c_desc ,
                      GMU_LVCOLUMN_CMDSTART , &c_cmdstart ,
                      GMU_LVCOLUMN_CMDSTOP , &c_cmdstop ,
                      GMU_LVCOLUMN_MONPROC , &c_monproc ,
                      GMU_LVCOLUMN_IFMON , &c_ifmon ,
                      GMU_LVCOLUMN_ICONFILE , &c_iconfile , -1 );

  c_entry = g_strjoin( "" , "entry-" , c_name , NULL );

  DEBUGMSG( "saving information for entry '%s' on keyfile...\n" , c_entry );
  g_key_file_set_boolean( config , c_entry , "ifenable" , c_ifenable );
  g_key_file_set_string( config , c_entry , "name" , c_name );
  g_key_file_set_string( config , c_entry , "desc" , c_desc );
  g_key_file_set_string( config , c_entry , "cmdstart" , c_cmdstart );
  g_key_file_set_string( config , c_entry , "cmdstop" , c_cmdstop );
  g_key_file_set_string( config , c_entry , "monproc" , c_monproc );
  g_key_file_set_boolean( config , c_entry , "ifmon" , c_ifmon );
  g_key_file_set_string( config , c_entry , "iconfile" , c_iconfile );
  DEBUGMSG( "information for entry '%s' saved on keyfile!\n" , c_entry );

  g_free( c_entry );
  g_free( c_name ); g_free( c_desc); g_free( c_cmdstart );
  g_free( c_cmdstop ); g_free( c_monproc ); g_free( c_iconfile );
  return FALSE;
}


void
gmu_cb_configwin_ok ( GtkWidget * config_window )
{
  GtkWidget *lv;
  GtkTreeModel *liststore;
  GtkTreeIter iter;
  GtkWidget *evbox;
  GtkWidget *main_menu;

  lv = g_object_get_data( G_OBJECT(config_window) , "listview" );
  liststore = gtk_tree_view_get_model( GTK_TREE_VIEW(lv) );

  /* drop old configuration */
  g_key_file_free( config );

  /* create a new configuration */
  config = g_key_file_new();

  /* fill the configuration with information in the listview */
  gtk_tree_model_foreach( liststore , gmu_cb_configwin_commit , NULL );

  /* update the main menu */
  evbox = g_object_get_data( G_OBJECT(config_window) , "tray_eventbox" );
  main_menu = g_object_get_data( G_OBJECT(evbox) , "main_menu" );
  gtk_widget_destroy( main_menu ); /* destroy old menu */
  main_menu = gmu_ui_mmenu_create(); /* generate new one */
  g_object_set_data( G_OBJECT(evbox) , "main_menu" , main_menu ); /* replace the old with the new */

  gtk_widget_destroy( config_window );
}


void
gmu_cb_configwin_edit ( GtkWidget * config_window )
{
  GtkWidget *lv;
  GtkTreeModel *liststore;
  GtkTreeSelection *listsel;
  GtkTreeIter iter;

  lv = g_object_get_data( G_OBJECT(config_window) , "listview" );
  listsel = gtk_tree_view_get_selection( GTK_TREE_VIEW(lv) );
  liststore = gtk_tree_view_get_model( GTK_TREE_VIEW(lv) );
  if ( gtk_tree_selection_get_selected( listsel , &liststore , &iter ) )
  {
    gmu_ui_configwin_modwin( GTK_WIDGET(config_window) , GMU_CONFIGWIN_EDIT );
  }
}


void
gmu_cb_configwin_remove ( GtkWidget * config_window )
{
  GtkWidget *lv;
  GtkTreeModel *liststore;
  GtkTreeSelection *listsel;
  GtkTreeIter iter;

  lv = GTK_WIDGET(g_object_get_data( G_OBJECT(config_window) , "listview" ));
  listsel = gtk_tree_view_get_selection( GTK_TREE_VIEW(lv) );
  liststore = gtk_tree_view_get_model( GTK_TREE_VIEW(lv) );
  if ( gtk_tree_selection_get_selected( listsel , &liststore , &iter ) )
  {
    GtkWidget *messagebox;
    gint result;
    messagebox = gtk_message_dialog_new( GTK_WINDOW(config_window) ,
                                         GTK_DIALOG_DESTROY_WITH_PARENT ,
                                         GTK_MESSAGE_QUESTION ,
                                         GTK_BUTTONS_YES_NO ,
                                         "Are you sure you want to remove\nthe selected entry?" );
    result = gtk_dialog_run( GTK_DIALOG(messagebox) );
    if ( result == GTK_RESPONSE_YES )
      gtk_list_store_remove( GTK_LIST_STORE(liststore) , &iter );
    gtk_widget_destroy( messagebox );
  }
}


void
gmu_cb_configwin_swap ( GtkWidget * button , gpointer config_window )
{
  GtkWidget *lv;
  GtkTreeModel *liststore;
  GtkTreeSelection *listsel;
  GtkTreeIter iter;

  lv = GTK_WIDGET(g_object_get_data( G_OBJECT(config_window) , "listview" ));
  listsel = gtk_tree_view_get_selection( GTK_TREE_VIEW(lv) );
  liststore = gtk_tree_view_get_model( GTK_TREE_VIEW(lv) );

  if ( gtk_tree_selection_get_selected( listsel , &liststore , &iter ) )
  {
    guint swapdire = GPOINTER_TO_UINT(g_object_get_data( G_OBJECT(button) , "swapdire" ));
    if ( swapdire == GMU_SWAPDIRE_UP ) /* move up */
    {
      GtkTreePath *treepath = gtk_tree_model_get_path( liststore, &iter );
      if ( gtk_tree_path_prev( treepath ) )
      {
         GtkTreeIter iter_prev;
         if ( gtk_tree_model_get_iter( liststore , &iter_prev , treepath ) )
         {
           gtk_list_store_swap( GTK_LIST_STORE(liststore) , &iter , &iter_prev );
         }
      }
      gtk_tree_path_free( treepath );
    }
    else /* move down */
    {
      GtkTreeIter iter_prev = iter;
      if ( gtk_tree_model_iter_next( liststore , &iter ) )
      {
        gtk_list_store_swap( GTK_LIST_STORE(liststore) , &iter , &iter_prev );
      }
    }
  }
}


void gmu_cb_configwin_toggle_ifenable ( gpointer liststore , gchar * path_str )
{
  GtkTreeModel *model = GTK_TREE_MODEL(liststore);
  GtkTreeIter iter;
  GtkTreePath *path = gtk_tree_path_new_from_string( path_str );
  gboolean toggled;

  gtk_tree_model_get_iter( model , &iter , path );
  gtk_tree_model_get( model , &iter ,
                      GMU_LVCOLUMN_IFENABLE , &toggled , -1);

  toggled ^= 1;
  gtk_list_store_set( GTK_LIST_STORE(model), &iter , GMU_LVCOLUMN_IFENABLE , toggled , -1);

  gtk_tree_path_free (path);
}


void gmu_cb_configwin_toggle_ifmon ( gpointer liststore , gchar * path_str )
{
  GtkTreeModel *model = GTK_TREE_MODEL(liststore);
  GtkTreeIter iter;
  GtkTreePath *path = gtk_tree_path_new_from_string( path_str );
  gboolean toggled;
  gchar *monproc = NULL;

  gtk_tree_model_get_iter( model , &iter , path );
  gtk_tree_model_get( model , &iter ,
                      GMU_LVCOLUMN_IFMON , &toggled ,
                      GMU_LVCOLUMN_MONPROC , &monproc , -1);

  /* check that the monproc entry field is not empty */
  if ( strcmp( monproc , "" ) )
  {
    toggled ^= 1;
    gtk_list_store_set( GTK_LIST_STORE(model), &iter , GMU_LVCOLUMN_IFMON , toggled , -1);
  }

  g_free( monproc );
  gtk_tree_path_free (path);
}


void
gmu_cb_configwin_modwin_browse ( GtkWidget * target_entry )
{
  GtkWidget *browse_dialog;
  GtkWidget *parent_window;

  parent_window = gtk_widget_get_toplevel( target_entry );
  if ( GTK_WIDGET_TOPLEVEL(parent_window) )
  {
    browse_dialog = gtk_file_chooser_dialog_new( _("gMUTOO - select file") ,
                                                 GTK_WINDOW(parent_window) ,
                                                 GTK_FILE_CHOOSER_ACTION_OPEN ,
                                                 GTK_STOCK_CANCEL , GTK_RESPONSE_CANCEL ,
                                                 GTK_STOCK_OPEN , GTK_RESPONSE_ACCEPT , NULL );

    if ( gtk_dialog_run(GTK_DIALOG(browse_dialog)) == GTK_RESPONSE_ACCEPT )
    {
      gchar *filename;
      filename = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER(browse_dialog) );
      gtk_entry_set_text( GTK_ENTRY(target_entry) , filename );
      DEBUGMSG( "selected file: %s\n" , filename );
      g_free( filename );
    }

    gtk_widget_destroy( browse_dialog );
  }
}


gboolean
gmu_cb_configwin_modwin_checkforequals( GtkTreeModel * store , GtkTreePath * path ,
                                        GtkTreeIter * iter , gpointer c_name )
{
  gchar * i_name;
  gtk_tree_model_get( store , iter ,
                      GMU_LVCOLUMN_NAME , &i_name , -1 );
  if ( !strcmp( c_name , i_name ) )
  {
    /* an existing entry with the same name has been found;
       to signal this, let's change c_name to a blank string "" */
    strcpy( c_name , "" );
    g_free( i_name );
    return TRUE; /* stop traversing the model */
  }
  g_free( i_name );
  return FALSE; /* keep traversing the model */
}


gint
gmu_cb_monitor_label_off ( gpointer status_label )
{
  gtk_label_set_markup( GTK_LABEL(status_label) , "<span font_desc=\"Sans 8\">status: <b>off</b></span>" );
  g_object_set_data( G_OBJECT(status_label) , "status" , GUINT_TO_POINTER(GMU_PROCSTATUS_OFF) );
  return 0;
}


gint
gmu_cb_monitor_label_on ( gpointer status_label )
{
  gtk_label_set_markup( GTK_LABEL(status_label) , "<span font_desc=\"Sans 8\">status: <b>on</b></span>" );
  g_object_set_data( G_OBJECT(status_label) , "status" , GUINT_TO_POINTER(GMU_PROCSTATUS_ON) );
  return 0;
}


void
gmu_cb_monitor_destroy ( gpointer monitor )
{
  gmu_mon_remove_file( monitor );
}
