/*
*
* Author: Giacomo Lozito <james@develia.org>, (C) 2005-2006
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or (at your
* option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*
*/

#include "cfg.h"


gint
gmu_cfg_read_file ( void )
{
  gchar * config_pathfile;

  config_pathfile = g_strjoin( NULL , g_get_home_dir() , "/.gmutoo/entries" , NULL );
  config = g_key_file_new();

  if ( !g_file_test( config_pathfile , G_FILE_TEST_EXISTS ) )
  {
    DEBUGMSG( "local config file '%s' not found, empty keyfile created!\n" , config_pathfile );
    g_free( config_pathfile );
    return 0;
  }

  DEBUGMSG( "local config file '%s' found, trying to read keyfile from it...\n" , config_pathfile );
  if ( !g_key_file_load_from_file( config , config_pathfile ,
                                   G_KEY_FILE_KEEP_COMMENTS , NULL ) )
  {
    DEBUGMSG( "local config file '%s' found, an error occurred during keyfile read!\n" , config_pathfile );
    g_free( config_pathfile );
    return -1;
  }
  else
  {
    DEBUGMSG( "local config file '%s' found, keyfile read successful!\n" , config_pathfile );
    g_free( config_pathfile );
    return 0;
  }
}


gint
gmu_cfg_write_file( GKeyFile *kf )
{
  gchar *config_string;
  gchar * config_pathfile;

  config_pathfile = g_strjoin( NULL , g_get_home_dir() , "/.gmutoo" , NULL );

  if ( !g_file_test( config_pathfile , G_FILE_TEST_IS_DIR ) )
  {
    DEBUGMSG( "local config directory '%s' not found\n" , config_pathfile );
    if ( g_file_test( config_pathfile , G_FILE_TEST_EXISTS ) )
    {
      DEBUGMSG( "it seems that '%s' exists but it is not a dir... config save aborted!\n" , config_pathfile );
      g_free( config_pathfile );
      return -1;
    }
    else
    {
      if ( g_mkdir( config_pathfile , 0755 ) == -1 )
      {
        DEBUGMSG( "unable to create local config dir '%s'... config save aborted!\n" , config_pathfile );
        g_free( config_pathfile );
        return -1;
      }
      #ifdef DEBUG
      else { DEBUGMSG( "local config directory '%s' created!\n" , config_pathfile ); }
      #endif /* DEBUG */
    }
  }

  g_free( config_pathfile );
  config_pathfile = g_strjoin( NULL , g_get_home_dir() , "/.gmutoo/entries" , NULL );
  config_string = g_key_file_to_data( kf , NULL , NULL );

  if ( g_file_set_contents( config_pathfile , config_string , -1 , NULL ) )
  {
    DEBUGMSG( "saved configuration to '%s'\n" , config_pathfile );
    g_free( config_pathfile );
    g_free( config_string );
    return 0;
  }
  else
  {
    DEBUGMSG( "an error occurred, unable to save configuration to '%s'\n" , config_pathfile );
    g_free( config_pathfile );
    g_free( config_string );
    return -1;
  }
}


gchar **
gmu_cfg_read_groups ( GKeyFile *kf , gsize *num )
{
  /* merely a wrapper around g_key_file_get_groups() */
  return g_key_file_get_groups( kf , num );
}


gchar *
gmu_cfg_read_string ( GKeyFile *kf , const gchar * group_name , const gchar * key_name )
{
  /* merely a wrapper around g_key_file_get_string() */
  return g_key_file_get_string( kf , group_name , key_name , NULL );
}


gchar *
gmu_cfg_read_string_with_default ( GKeyFile *kf , const gchar * group_name ,
                                   const gchar * key_name , gchar * default_value )
{
  /* more than a wrapper around g_key_file_get_string() */
  gchar * value = g_key_file_get_string( kf , group_name , key_name , NULL );
  if ( !value )
  {
    DEBUGMSG( "key '%s' for group '%s' not found!\n" , key_name , group_name );
    value = g_strdup( default_value );
  }
  #ifdef DEBUG
  else { DEBUGMSG( "key '%s' for group '%s' found: '%s'\n" , key_name , group_name , value ); }
  #endif /* DEBUG */
  return value;
}


gboolean
gmu_cfg_read_boolean_with_default ( GKeyFile *kf , const gchar * group_name ,
                                    const gchar * key_name , gboolean default_value )
{
  GError * gerr = NULL;
  /* more than a wrapper around g_key_file_get_string() */
  gboolean value = g_key_file_get_boolean( kf , group_name , key_name , &gerr );
  if ( gerr != NULL )
  {
    DEBUGMSG( "key '%s' for group '%s' not found!\n" , key_name , group_name );
    value = default_value;
    g_error_free( gerr );
  }
  #ifdef DEBUG
  else { DEBUGMSG( "key '%s' for group '%s' found: '%i'\n" , key_name , group_name , value ); }
  #endif /* DEBUG */
  return value;
}
