/*
*
* Author: Giacomo Lozito <james@develia.org>, (C) 2005-2006
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or (at your
* option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*
*/

#include "ui.h"
#include "cb.h"
#include "cfg.h"
#include "mon.h"
#include "images/gmutoo_icon.xpm"

#define ICON_PIXBUF(x,y) gtk_icon_theme_load_icon( gtk_icon_theme_get_default() , x , y , 0 , NULL )


void
gmu_ui_icons_init ( void )
{
  GdkPixbuf *normal_pixbuf, *scaled_pixbuf;

  normal_pixbuf = gdk_pixbuf_new_from_xpm_data( (const char**)gmutoo_icon_xpm );
  scaled_pixbuf = gdk_pixbuf_scale_simple( normal_pixbuf , 32 , 32 , GDK_INTERP_BILINEAR );
  gtk_icon_theme_add_builtin_icon( "gmutoo-main" , 32 , scaled_pixbuf );
  g_object_unref( scaled_pixbuf );
  scaled_pixbuf = gdk_pixbuf_scale_simple( normal_pixbuf , 22 , 22 , GDK_INTERP_BILINEAR );
  gtk_icon_theme_add_builtin_icon( "gmutoo-main" , 22 , scaled_pixbuf );
  g_object_unref( scaled_pixbuf );

  g_object_unref( normal_pixbuf );
}


EggTrayIcon *
gmu_ui_trayapp ( void )
{
  EggTrayIcon *tray_applet = NULL;

  tray_applet = egg_tray_icon_new( "gmutoo" );

  g_signal_connect( G_OBJECT(tray_applet) , "destroy" ,
                    G_CALLBACK(gmu_cb_trayapp_destroy) , NULL );

  gtk_widget_show( GTK_WIDGET(tray_applet) );

  return tray_applet;
}


void
gmu_ui_trayapp_start ( EggTrayIcon * tray_applet )
{
  GtkWidget *tray_evbox, *tray_image;
  GtkWidget *tray_mmenu, *tray_cmenu;
  GdkPixbuf *pixbuf;

  pixbuf = ICON_PIXBUF( "gmutoo-main" , 22 );
  tray_image = gtk_image_new_from_pixbuf( pixbuf );
  g_object_unref( pixbuf );

  tray_evbox = gtk_event_box_new();

  tray_mmenu = gmu_ui_mmenu_create();
  tray_cmenu = gmu_ui_cmenu_create( tray_evbox );

  g_object_set_data( G_OBJECT(tray_evbox) , "main_menu" , tray_mmenu );
  g_object_set_data( G_OBJECT(tray_evbox) , "config_menu" , tray_cmenu );
  g_signal_connect( G_OBJECT(tray_evbox) , "button-press-event" ,
                    G_CALLBACK(gmu_cb_trayapp_bpress) , NULL );
  g_signal_connect( G_OBJECT(tray_evbox) , "destroy" ,
                    G_CALLBACK(gmu_cb_cmenu_quit) , NULL );

  gtk_container_add( GTK_CONTAINER(tray_evbox), tray_image );
  gtk_container_add( GTK_CONTAINER(tray_applet), tray_evbox );

  gtk_widget_show_all( GTK_WIDGET(tray_applet) );

  return;
}


GtkWidget *
gmu_ui_mmenu_create ( void )
{
  GtkWidget *menu = NULL;
  GdkPixbuf *default_pixbuf;
  gchar **config_group;
  gsize config_groups_num = 0;
  gint i = 0;
  gint i_added = 0;

  menu = gtk_menu_new();
  config_group = gmu_cfg_read_groups( config , &config_groups_num );
  DEBUGMSG( "number of valid keyfile groups: %i\n" , config_groups_num );
  default_pixbuf = ICON_PIXBUF( "gmutoo-main" , 32 );

  for ( i = 0 ; i < config_groups_num ; i++ )
  {
    GtkWidget *menuitem;
    GtkWidget *image = NULL;
    GtkWidget *hbox, *vbox, *main_label, *status_label;
    gchar *markup_text;
    gchar *entry_name, *entry_cmdstart, *entry_cmdstop;
    gchar *entry_monproc, *entry_iconfile;
    gboolean entry_ifenable, entry_ifmon;

    if ( strncmp( "entry-" , config_group[i] , 6 ) )
    {
      /* it is not a menu entry, skip */
      DEBUGMSG( "group '%s' is not a menu entry! skipping this group\n" , config_group[i] );
      continue;
    }

    DEBUGMSG( "processing keys for group '%s'...\n" , config_group[i] );
    entry_ifenable = gmu_cfg_read_boolean_with_default( config , config_group[i] , "ifenable" , FALSE );
    if ( entry_ifenable == FALSE )
    {
      DEBUGMSG( "group '%s' has been disabled! skipping this group\n" , config_group[i] );
      continue;
    }

    entry_name = gmu_cfg_read_string( config , config_group[i] , "name" );
    if ( !entry_name )
    {
      DEBUGMSG( "key 'name' for group '%s' not found! skipping this group\n" , config_group[i] );
      continue;
    }
    #ifdef DEBUG
    else { DEBUGMSG( "key 'name' for group '%s' found: '%s'\n" , config_group[i] , entry_name ); }
    #endif /* DEBUG */

    if ( i_added > 0 )
    {
      GtkWidget *menusep = gtk_separator_menu_item_new();
      gtk_menu_shell_append( GTK_MENU_SHELL(menu) , menusep );
    }

    entry_cmdstart = gmu_cfg_read_string_with_default( config , config_group[i] , "cmdstart" , "" );
    entry_cmdstop = gmu_cfg_read_string_with_default( config , config_group[i] , "cmdstop" , "" );
    entry_monproc = gmu_cfg_read_string_with_default( config , config_group[i] , "monproc" , "" );
    entry_iconfile = gmu_cfg_read_string_with_default( config , config_group[i] , "iconfile" , "" );
    #ifdef USE_GAMIN
    entry_ifmon = gmu_cfg_read_boolean_with_default( config , config_group[i] , "ifmon" , FALSE );
    #else
    entry_ifmon = FALSE; /* always FALSE if USE_GAMIN is not defined */
    #endif

    menuitem = gtk_menu_item_new();
    hbox = gtk_hbox_new( FALSE , 8 );
    gtk_container_add( GTK_CONTAINER(menuitem) , hbox );

    if (( strcmp( entry_iconfile , "" ) ) &&
        ( g_file_test( entry_iconfile , G_FILE_TEST_EXISTS ) ))
    {
      /* the file specified in the iconfile field exists,
         let's try to create a GdkPixbuf out of it*/
      GdkPixbuf *normal_pixbuf;
      GError *error = NULL;

      normal_pixbuf = gdk_pixbuf_new_from_file( entry_iconfile , &error );
      if ( normal_pixbuf != NULL )
      {
        GdkPixbuf *scaled_pixbuf;
        /* scale the pixbuf to 32x32 */
        scaled_pixbuf = gdk_pixbuf_scale_simple( normal_pixbuf , 32 , 32 , GDK_INTERP_BILINEAR );
        image = gtk_image_new_from_pixbuf( scaled_pixbuf );
        g_object_unref( scaled_pixbuf );
        g_object_unref( normal_pixbuf );
      }
      else
      {
        g_warning( "Unable to load icon file '%s' - error message: %s" ,
                   entry_iconfile , error->message );
      }
    }
    /* check if an image has been set; if not, go with default one */
    if ( image == NULL )
    {
      image = gtk_image_new_from_pixbuf( default_pixbuf );
    }

    gtk_box_pack_start( GTK_BOX(hbox) , image , FALSE , TRUE , 3 );

    /* add the vertical box */
    vbox = gtk_vbox_new( FALSE , 0 );
    gtk_box_pack_start( GTK_BOX(hbox) , vbox , TRUE , TRUE , 3 );

    /* add the main label */
    main_label = gtk_label_new( "" );
    gtk_misc_set_alignment( GTK_MISC(main_label) , 0 , 0.5 );
    markup_text = g_markup_printf_escaped ( "<span font_desc=\"Sans 18\">%s</span>" , entry_name );
    gtk_label_set_markup( GTK_LABEL(main_label) , markup_text );
    g_free( markup_text );
    gtk_box_pack_start( GTK_BOX(vbox) , main_label , TRUE , TRUE , 1 );

    /* add the status label */
    status_label = gtk_label_new( "" );
    gtk_misc_set_alignment( GTK_MISC(status_label) , 0 , 0.5 );
    /* if monitoring is disabled, add a 'not monitored' tag */
    if ( entry_ifmon == FALSE )
    {
      markup_text = g_markup_printf_escaped ( "<span font_desc=\"Sans 8\">not monitored</span>" );
    }
    else
    {
      markup_text = g_markup_printf_escaped ( "<span font_desc=\"Sans 8\">status: <b>%s</b></span>" , "off" );
      g_object_set_data( G_OBJECT(status_label) , "status" , GUINT_TO_POINTER(GMU_PROCSTATUS_OFF) );
    }
    gtk_label_set_markup( GTK_LABEL(status_label) , markup_text );
    g_free( markup_text );
    gtk_box_pack_start( GTK_BOX(vbox) , status_label , TRUE , TRUE , 1 );

    /* if monitoring is disabled, add a start/stop submenu */
    if ( entry_ifmon == FALSE )
    {
      GtkWidget *submenu;
      GtkWidget *menuitem_start, *menuitem_stop;

      submenu = gtk_menu_new();
      menuitem_start = gtk_image_menu_item_new_with_label( "start" );
      gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(menuitem_start) ,
                                     gtk_image_new_from_stock( GTK_STOCK_MEDIA_PLAY , GTK_ICON_SIZE_MENU ) );
      g_signal_connect_swapped( G_OBJECT(menuitem_start) , "activate" ,
                                G_CALLBACK(gmu_cb_mmenu_execute_generic) , entry_cmdstart );
      menuitem_stop = gtk_image_menu_item_new_with_label( "stop" );
      gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(menuitem_stop) ,
                                     gtk_image_new_from_stock( GTK_STOCK_MEDIA_STOP , GTK_ICON_SIZE_MENU ) );
      g_signal_connect_swapped( G_OBJECT(menuitem_stop) , "activate" ,
                                G_CALLBACK(gmu_cb_mmenu_execute_generic) , entry_cmdstop );
      gtk_menu_shell_append( GTK_MENU_SHELL(submenu) , menuitem_start );
      gtk_menu_shell_append( GTK_MENU_SHELL(submenu) , menuitem_stop );
      gtk_menu_item_set_submenu( GTK_MENU_ITEM(menuitem) , submenu );
    }
    else /* monitoring enabled! */
    {
      if ( strcmp(entry_monproc , "") )
      {
        gpointer monitor = NULL;
        /* add a new file monitor */
        if ( gmu_mon_add_file( entry_monproc , &monitor ,
                               gmu_cb_monitor_label_off ,
                               gmu_cb_monitor_label_on ,
                               status_label ) == 0 )
        {
          /* create a routine that closes the file monitor on destroy */
          g_signal_connect_swapped( G_OBJECT(menuitem) , "destroy" ,
                                    G_CALLBACK(gmu_cb_monitor_destroy) , monitor );
          /* attach start and stop command strings to the status label */
          g_object_set_data( G_OBJECT(status_label) , "cmdstart" , entry_cmdstart );
          g_object_set_data( G_OBJECT(status_label) , "cmdstop" , entry_cmdstop );
          g_signal_connect_swapped( G_OBJECT(menuitem) , "activate" ,
                                    G_CALLBACK(gmu_cb_mmenu_execute_monitored) , status_label );
        }
      }
    }

    ++i_added;
    gtk_menu_shell_append( GTK_MENU_SHELL(menu) , menuitem );
  }

  g_object_unref( default_pixbuf );
  g_strfreev( config_group );

  gtk_widget_show_all( menu );
  return menu;
}


GtkWidget *
gmu_ui_cmenu_create ( GtkWidget * evbox )
{
  GtkWidget *menu;
  GtkWidget *menuitem;

  menu = gtk_menu_new();

  menuitem = gtk_image_menu_item_new_with_label( "Configuration..." );
  gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(menuitem) ,
                                 gtk_image_new_from_stock( GTK_STOCK_PREFERENCES , GTK_ICON_SIZE_MENU ) );
  g_signal_connect_swapped( G_OBJECT(menuitem) , "activate" , G_CALLBACK(gmu_cb_cmenu_config) , evbox );
  gtk_menu_shell_append( GTK_MENU_SHELL(menu) , menuitem );

  menuitem = gtk_separator_menu_item_new();
  gtk_menu_shell_append( GTK_MENU_SHELL(menu) , menuitem );

  menuitem = gtk_image_menu_item_new_with_label( "Quit" );
  gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(menuitem) ,
                                 gtk_image_new_from_stock( GTK_STOCK_QUIT , GTK_ICON_SIZE_MENU ) );
  g_signal_connect_swapped( G_OBJECT(menuitem) , "activate" , G_CALLBACK(gtk_widget_destroy) , evbox );
  gtk_menu_shell_append( GTK_MENU_SHELL(menu) , menuitem );

  gtk_widget_show_all( GTK_WIDGET(menu) );
  return menu;
}


void
gmu_ui_warningbox( GtkWindow * parent , const gchar * message )
{
  GtkWidget *messagebox;
  messagebox = gtk_message_dialog_new( parent , GTK_DIALOG_DESTROY_WITH_PARENT ,
                                       GTK_MESSAGE_WARNING , GTK_BUTTONS_OK , message );
  gtk_dialog_run( GTK_DIALOG(messagebox) );
  gtk_widget_destroy( messagebox );
}


void
gmu_ui_configwin_render_cell_img ( GtkTreeViewColumn * col , GtkCellRenderer * cr ,
                                   GtkTreeModel * store , GtkTreeIter * iter , gpointer data )
{
  gchar *entry_iconfile;
  GdkPixbuf *scaled_pixbuf = NULL;

  gtk_tree_model_get( store , iter ,
                      GMU_LVCOLUMN_ICONFILE , &entry_iconfile , -1 );

  if (( strcmp( entry_iconfile , "" ) ) &&
        ( g_file_test( entry_iconfile , G_FILE_TEST_EXISTS ) ))
  {
    /* the file specified in the iconfile field exists,
       let's try to create a GdkPixbuf out of it*/
    GdkPixbuf *normal_pixbuf;
    GError *error = NULL;

    normal_pixbuf = gdk_pixbuf_new_from_file( entry_iconfile , &error );
    if ( normal_pixbuf != NULL )
    {
      /* scale the pixbuf to 32x32 */
      scaled_pixbuf = gdk_pixbuf_scale_simple( normal_pixbuf , 32 , 32 , GDK_INTERP_BILINEAR );
      g_object_unref( normal_pixbuf );
    }
    else
    {
      g_warning( "Unable to load icon file '%s' - error message: %s" ,
                 entry_iconfile , error->message );
    }
  }
  if ( scaled_pixbuf == NULL ) /* go with default one */
  {
    scaled_pixbuf = ICON_PIXBUF( "gmutoo-main" , 32 );
  }

  g_object_set( G_OBJECT(cr) , "pixbuf" , scaled_pixbuf , NULL );
  g_free( entry_iconfile );
  g_object_unref( scaled_pixbuf );
}


void
gmu_ui_configwin_render_cell_name ( GtkTreeViewColumn * col , GtkCellRenderer * cr ,
                                    GtkTreeModel * store , GtkTreeIter * iter , gpointer data )
{
  gchar *entry_name, *entry_desc;
  gchar *markup_text;
  gtk_tree_model_get( store , iter ,
                      GMU_LVCOLUMN_NAME , &entry_name ,
                      GMU_LVCOLUMN_DESC , &entry_desc , -1 );
  markup_text = g_markup_printf_escaped ( "<span font_desc=\"Sans 18\">%s</span>\n"
                                          "<span font_desc=\"Sans 8\" weight=\"light\">%s</span>" ,
                                          entry_name , entry_desc );
  g_object_set( G_OBJECT(cr) , "markup" , markup_text , NULL );
  g_free( markup_text );
  g_free( entry_desc );
  g_free( entry_name );
}


void
gmu_ui_configwin ( GtkWidget * evbox )
{
  static GtkWidget * config_window = NULL;

  if ( !config_window )
  {
    GdkPixbuf *config_window_icon;
    GtkWidget *vbox;
    GtkWidget *hseparator;
    GtkWidget *hbuttonbox_bottom, *button_ok, *button_cancel;
    GtkWidget *hbox_lv, *lv, *lv_frame, *lv_sw;
    GtkWidget *vbox_controls, *vbox_edit, *button_add, *button_edit, *button_remove;
    GtkWidget *vbox_swap, *button_swapup, *button_swapdown;
    GtkWidget *button_about;
    GdkGeometry cw_hints;
    GtkCellRenderer *lv_cr_image, *lv_cr_text, *lv_cr_toggle_ifenable, *lv_cr_toggle_ifmon;
    GtkTreeViewColumn *lv_col_name, *lv_col_ifenable, *lv_col_ifmon;
    GtkListStore *liststore;
    GtkTreeIter iter;
    GtkTooltips *tooltips;
    gsize config_groups_num = 0;
    gchar **config_group;
    gint i = 0;

    config_window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_type_hint( GTK_WINDOW(config_window) , GDK_WINDOW_TYPE_HINT_DIALOG );
    g_signal_connect( G_OBJECT(config_window) , "destroy" , G_CALLBACK(gtk_widget_destroyed) , &config_window );
    gtk_window_set_title( GTK_WINDOW(config_window) , _("gMUTOO - configuration") );
    g_object_set_data( G_OBJECT(config_window) , "tray_eventbox" , evbox );
    config_window_icon = ICON_PIXBUF( "gmutoo-main" , 32 );
    gtk_window_set_icon( GTK_WINDOW(config_window) , config_window_icon );
    g_object_unref( config_window_icon );
    gtk_container_set_border_width( GTK_CONTAINER(config_window), 10 );
    /* minimum widht: 500 pixel; minimum height: 250 pixel; */
    cw_hints.min_width = 500; cw_hints.min_height = 360;
    gtk_window_set_geometry_hints( GTK_WINDOW(config_window) , GTK_WIDGET(config_window) ,
                                   &cw_hints , GDK_HINT_MIN_SIZE );

    tooltips = gtk_tooltips_new();

    vbox = gtk_vbox_new( FALSE , 0 );
    gtk_container_add( GTK_CONTAINER(config_window) , vbox );

    hbox_lv = gtk_hbox_new( FALSE , 10 );

    /* listview - store fill */
    config_group = gmu_cfg_read_groups( config , &config_groups_num );
    DEBUGMSG( "number of valid keyfile groups: %i\n" , config_groups_num );

    liststore = gtk_list_store_new( GMU_NUM_LVCOLUMNS , G_TYPE_STRING , G_TYPE_STRING ,
                                    G_TYPE_STRING , G_TYPE_STRING , G_TYPE_STRING ,
                                    G_TYPE_BOOLEAN , G_TYPE_STRING , G_TYPE_BOOLEAN );

    for ( i = 0 ; i < config_groups_num ; i++ )
    {
      gchar *entry_name, *entry_desc, *entry_cmdstart;
      gchar *entry_cmdstop, *entry_monproc, *entry_iconfile;
      gboolean entry_ifmon, entry_ifenable;
      DEBUGMSG( "retrieving keys for group '%s'...\n" , config_group[i] );
      entry_name = gmu_cfg_read_string( config , config_group[i] , "name" );
      if ( !entry_name )
      {
        DEBUGMSG( "key 'name' for group '%s' not found! skipping this group\n" , config_group[i] );
        continue;
      }
      #ifdef DEBUG
      else { DEBUGMSG( "key 'name' for group '%s' found: '%s'\n" , config_group[i] , entry_name ); }
      #endif /* DEBUG */

      entry_ifenable = gmu_cfg_read_boolean_with_default( config , config_group[i] , "ifenable" , FALSE );
      entry_desc = gmu_cfg_read_string_with_default( config , config_group[i] , "desc" , "" );
      entry_cmdstart = gmu_cfg_read_string_with_default( config , config_group[i] , "cmdstart" , "" );
      entry_cmdstop = gmu_cfg_read_string_with_default( config , config_group[i] , "cmdstop" , "" );
      entry_monproc = gmu_cfg_read_string_with_default( config , config_group[i] , "monproc" , "" );
      entry_ifmon = gmu_cfg_read_boolean_with_default( config , config_group[i] , "ifmon" , FALSE );
      entry_iconfile = gmu_cfg_read_string_with_default( config , config_group[i] , "iconfile" , "" );

      gtk_list_store_append( liststore , &iter );
      gtk_list_store_set( liststore , &iter ,
                          GMU_LVCOLUMN_IFENABLE , entry_ifenable ,
                          GMU_LVCOLUMN_NAME , entry_name ,
                          GMU_LVCOLUMN_DESC , entry_desc ,
                          GMU_LVCOLUMN_CMDSTART , entry_cmdstart ,
                          GMU_LVCOLUMN_CMDSTOP , entry_cmdstop ,
                          GMU_LVCOLUMN_MONPROC , entry_monproc ,
                          GMU_LVCOLUMN_IFMON , entry_ifmon ,
                          GMU_LVCOLUMN_ICONFILE , entry_iconfile ,
                          -1 );

      g_free( entry_iconfile ); g_free( entry_monproc );
      g_free( entry_cmdstop ); g_free( entry_cmdstart );
      g_free( entry_desc); g_free( entry_name );
    }
    g_strfreev( config_group );

    /* listview - control */
    lv = gtk_tree_view_new_with_model( GTK_TREE_MODEL(liststore) );
    g_object_unref( liststore );
    /* grant quick access to the listview pointer by having the window pointer */
    g_object_set_data( G_OBJECT(config_window) , "listview" , lv );
    gtk_tree_view_set_rules_hint( GTK_TREE_VIEW(lv) , TRUE );
    gtk_tree_selection_set_mode( gtk_tree_view_get_selection(GTK_TREE_VIEW(lv)) , GTK_SELECTION_SINGLE );
    lv_cr_text = gtk_cell_renderer_text_new();
    lv_cr_image = gtk_cell_renderer_pixbuf_new();
    lv_cr_toggle_ifenable = gtk_cell_renderer_toggle_new();
    g_signal_connect_swapped( lv_cr_toggle_ifenable , "toggled" ,
                              G_CALLBACK(gmu_cb_configwin_toggle_ifenable) , liststore );
    gtk_cell_renderer_toggle_set_radio( GTK_CELL_RENDERER_TOGGLE(lv_cr_toggle_ifenable) , FALSE );
    gtk_cell_renderer_toggle_set_active( GTK_CELL_RENDERER_TOGGLE(lv_cr_toggle_ifenable) , TRUE );
    lv_cr_toggle_ifmon = gtk_cell_renderer_toggle_new();
    g_signal_connect_swapped( lv_cr_toggle_ifmon , "toggled" ,
                              G_CALLBACK(gmu_cb_configwin_toggle_ifmon) , liststore );
    gtk_cell_renderer_toggle_set_radio( GTK_CELL_RENDERER_TOGGLE(lv_cr_toggle_ifmon) , FALSE );
    gtk_cell_renderer_toggle_set_active( GTK_CELL_RENDERER_TOGGLE(lv_cr_toggle_ifmon) , TRUE );

    lv_col_ifenable = gtk_tree_view_column_new_with_attributes( "On" , lv_cr_toggle_ifenable ,
                                                            "active" , GMU_LVCOLUMN_IFENABLE , NULL );
    gtk_tree_view_column_set_expand( lv_col_ifenable , FALSE );
    gtk_tree_view_append_column( GTK_TREE_VIEW(lv), lv_col_ifenable );
    lv_col_name = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title( lv_col_name , "Menu Entry" );
    gtk_tree_view_column_set_expand( lv_col_name , TRUE );
    gtk_tree_view_append_column( GTK_TREE_VIEW(lv), lv_col_name );
    gtk_tree_view_column_set_spacing( lv_col_name , 10 );
    gtk_tree_view_column_pack_start( lv_col_name , lv_cr_image , FALSE );
    gtk_tree_view_column_pack_start( lv_col_name , lv_cr_text , TRUE );
    gtk_tree_view_column_set_cell_data_func( lv_col_name , lv_cr_image ,
                                             gmu_ui_configwin_render_cell_img , NULL , NULL );
    gtk_tree_view_column_set_cell_data_func( lv_col_name , lv_cr_text ,
                                             gmu_ui_configwin_render_cell_name , NULL , NULL );
    lv_col_ifmon = gtk_tree_view_column_new_with_attributes( "Mon" , lv_cr_toggle_ifmon ,
                                                             "active" , GMU_LVCOLUMN_IFMON , NULL );
    gtk_tree_view_column_set_expand( lv_col_ifmon , FALSE );
    gtk_tree_view_append_column( GTK_TREE_VIEW(lv), lv_col_ifmon );

    lv_sw = gtk_scrolled_window_new( NULL , NULL );
    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(lv_sw),
                                    GTK_POLICY_NEVER , GTK_POLICY_ALWAYS );
    gtk_container_add( GTK_CONTAINER(lv_sw) , lv );
    lv_frame = gtk_frame_new( NULL );
    gtk_container_add( GTK_CONTAINER(lv_frame) , lv_sw );

    gtk_box_pack_start( GTK_BOX(hbox_lv) , lv_frame , TRUE , TRUE , 0 );

    /* vertical listview buttonbox */
    vbox_controls = gtk_vbox_new( FALSE , 10 );

    vbox_edit = gtk_vbox_new( TRUE , 0 );
    button_add = gtk_button_new();
    gtk_button_set_image( GTK_BUTTON(button_add) ,
                          gtk_image_new_from_stock( GTK_STOCK_ADD , GTK_ICON_SIZE_BUTTON ) );
    g_signal_connect_swapped( G_OBJECT(button_add) , "clicked" ,
                              G_CALLBACK(gmu_cb_configwin_add) , config_window );
    gtk_box_pack_start( GTK_BOX(vbox_edit) , button_add , FALSE , FALSE , 0 );
    gtk_tooltips_set_tip( tooltips , button_add , "Add a new entry" , "" );
    button_edit = gtk_button_new();
    gtk_button_set_image( GTK_BUTTON(button_edit) ,
                          gtk_image_new_from_stock( GTK_STOCK_EDIT , GTK_ICON_SIZE_BUTTON ) );
    g_signal_connect_swapped( G_OBJECT(button_edit) , "clicked" ,
                              G_CALLBACK(gmu_cb_configwin_edit) , config_window );
    gtk_box_pack_start( GTK_BOX(vbox_edit) , button_edit , FALSE , FALSE , 0 );
    gtk_tooltips_set_tip( tooltips , button_edit , "Edit the selected entry" , "" );
    button_remove = gtk_button_new();
    gtk_button_set_image( GTK_BUTTON(button_remove) ,
                          gtk_image_new_from_stock( GTK_STOCK_REMOVE , GTK_ICON_SIZE_BUTTON ) );
    g_signal_connect_swapped( G_OBJECT(button_remove) , "clicked" ,
                              G_CALLBACK(gmu_cb_configwin_remove) , config_window );
    gtk_box_pack_start( GTK_BOX(vbox_edit) , button_remove , FALSE , FALSE , 0 );
    gtk_tooltips_set_tip( tooltips , button_remove , "Remove the selected entry" , "" );
    gtk_box_pack_start( GTK_BOX(vbox_controls) , vbox_edit , FALSE , FALSE , 0 );

    vbox_swap = gtk_vbox_new( TRUE , 0 );
    button_swapup = gtk_button_new();
    gtk_button_set_image( GTK_BUTTON(button_swapup) ,
                          gtk_image_new_from_stock( GTK_STOCK_GO_UP , GTK_ICON_SIZE_BUTTON ) );
    g_object_set_data( G_OBJECT(button_swapup) , "swapdire" , GUINT_TO_POINTER(GMU_SWAPDIRE_UP) );
    g_signal_connect( G_OBJECT(button_swapup) , "clicked" ,
                      G_CALLBACK(gmu_cb_configwin_swap) , config_window );
    gtk_box_pack_start( GTK_BOX(vbox_swap) , button_swapup , FALSE , FALSE , 0 );
    gtk_tooltips_set_tip( tooltips , button_swapup , "Move up the selected entry" , "" );
    button_swapdown = gtk_button_new();
    gtk_button_set_image( GTK_BUTTON(button_swapdown) ,
                          gtk_image_new_from_stock( GTK_STOCK_GO_DOWN , GTK_ICON_SIZE_BUTTON ) );
    g_object_set_data( G_OBJECT(button_swapdown) , "swapdire" , GUINT_TO_POINTER(GMU_SWAPDIRE_DOWN) );
    g_signal_connect( G_OBJECT(button_swapdown) , "clicked" ,
                      G_CALLBACK(gmu_cb_configwin_swap) , config_window );
    gtk_box_pack_start( GTK_BOX(vbox_swap) , button_swapdown , FALSE , FALSE , 0 );
    gtk_tooltips_set_tip( tooltips , button_swapdown , "Move down the selected entry" , "" );
    gtk_box_pack_start( GTK_BOX(vbox_controls) , vbox_swap , FALSE , FALSE , 0 );

    button_about = gtk_button_new();
    gtk_button_set_image( GTK_BUTTON(button_about) ,
                          gtk_image_new_from_stock( GTK_STOCK_ABOUT , GTK_ICON_SIZE_BUTTON ) );
    g_signal_connect_swapped( G_OBJECT(button_about) , "clicked" ,
                              G_CALLBACK(gmu_cb_configwin_about) , config_window );
    gtk_box_pack_start( GTK_BOX(vbox_controls) , button_about , FALSE , FALSE , 0 );
    gtk_tooltips_set_tip( tooltips , button_about , "About gMUTOO" , "" );

    gtk_box_pack_start( GTK_BOX(hbox_lv) , vbox_controls , FALSE , FALSE , 0 );
    gtk_box_pack_start( GTK_BOX(vbox) , hbox_lv , TRUE , TRUE , 2 );

    /* horizontal separator and buttons */
    hseparator = gtk_hseparator_new();
    gtk_box_pack_start( GTK_BOX(vbox) , hseparator , FALSE , FALSE , 4 );

    hbuttonbox_bottom = gtk_hbutton_box_new();
    gtk_button_box_set_layout( GTK_BUTTON_BOX(hbuttonbox_bottom) , GTK_BUTTONBOX_END );
    button_cancel = gtk_button_new_from_stock( GTK_STOCK_CANCEL );
    g_signal_connect_swapped( G_OBJECT(button_cancel) , "clicked" ,
                              G_CALLBACK(gtk_widget_destroy) , config_window );
    gtk_container_add( GTK_CONTAINER(hbuttonbox_bottom) , button_cancel );
    button_ok = gtk_button_new_from_stock( GTK_STOCK_OK );
    g_signal_connect_swapped( G_OBJECT(button_ok) , "clicked" ,
                              G_CALLBACK(gmu_cb_configwin_ok) , config_window );
    gtk_container_add( GTK_CONTAINER(hbuttonbox_bottom) , button_ok );
    gtk_box_pack_start( GTK_BOX(vbox) , hbuttonbox_bottom , FALSE , FALSE , 0 );

    gtk_widget_show_all( config_window );
  }
  else
  {
    gdk_window_raise( config_window->window );
  }
}


void
gmu_ui_configwin_modwin ( GtkWidget * parent_window , gint action )
{
  GtkWidget *config_modwindow, *table;
  GtkWidget *name_label, *name_entry, *desc_label, *desc_entry, *cmdstart_label, *cmdstart_entry, *cmdstart_browseb;
  GtkWidget *cmdstop_label, *cmdstop_entry, *cmdstop_browseb, *monproc_label, *monproc_entry;
  GtkWidget *monproc_browseb, *iconfile_label, *iconfile_entry, *iconfile_browseb;
  GdkGeometry aw_hints;
  GtkTreeView *ext_lv;
  GtkTreeModel *ext_liststore;
  GtkTreeIter ext_iter;
  gboolean done = FALSE;
  GtkTooltips *tooltips;

  config_modwindow = gtk_dialog_new();
  gtk_window_set_modal( GTK_WINDOW(config_modwindow) , TRUE );
  gtk_window_set_transient_for( GTK_WINDOW(config_modwindow) , GTK_WINDOW(parent_window) );
  tooltips = gtk_tooltips_new();

  aw_hints.min_width = 400; aw_hints.min_height = -1;
  gtk_window_set_geometry_hints( GTK_WINDOW(config_modwindow) , GTK_WIDGET(config_modwindow) ,
                                 &aw_hints , GDK_HINT_MIN_SIZE );

  table = gtk_table_new( 6 , 3 , FALSE );
  gtk_container_set_border_width( GTK_CONTAINER(table), 5 );
  /* grant quick access to the listview of the parent window by having the table pointer */
  g_object_set_data( G_OBJECT(table) , "listview" ,
                     g_object_get_data( G_OBJECT(parent_window) , "listview" ) );
  name_label = gtk_label_new( _("Name:") );
  gtk_misc_set_alignment( GTK_MISC(name_label) , 0 , 0.5 );
  name_entry = gtk_entry_new();
  gtk_entry_set_max_length( GTK_ENTRY(name_entry) , 20 ); /* name max length: 20 chars */
  gtk_tooltips_set_tip( tooltips , name_entry , "The NAME field is mandatory; it will be displayed "
                        "in the menu and has a max length of 20 chars." , "" );
  gtk_table_attach( GTK_TABLE(table) , name_label , 0 , 1 , 0 , 1 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 0 );
  gtk_table_attach( GTK_TABLE(table) , name_entry , 1 , 3 , 0 , 1 ,
                    GTK_FILL | GTK_EXPAND , GTK_FILL | GTK_EXPAND , 2 , 4 );
  desc_label = gtk_label_new( _("Descr.:") );
  gtk_misc_set_alignment( GTK_MISC(desc_label) , 0 , 0.5 );
  desc_entry = gtk_entry_new();
  gtk_entry_set_max_length( GTK_ENTRY(desc_entry) , 30 ); /* description max length: 30 chars */
  gtk_tooltips_set_tip( tooltips , desc_entry , "The DESCRIPTION field is optional and has a max length of "
                        "30 chars; actually it is displayed only in the configuration window (it will "
                        "eventually become more useful someday)." , "" );
  gtk_table_attach( GTK_TABLE(table) , desc_label , 0 , 1 , 1 , 2 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 0 );
  gtk_table_attach( GTK_TABLE(table) , desc_entry , 1 , 3 , 1 , 2 ,
                    GTK_FILL | GTK_EXPAND , GTK_FILL | GTK_EXPAND , 2 , 4 );
  cmdstart_label = gtk_label_new( _("Start CMD:") );
  gtk_misc_set_alignment( GTK_MISC(cmdstart_label) , 0 , 0.5 );
  cmdstart_entry = gtk_entry_new();
  gtk_tooltips_set_tip( tooltips , cmdstart_entry , "The CMDSTART field is optional (but obviously recommended); "
                        "the command MUST be specified with absolute path (otherwise it won't work); "
                        "the command specified here will be launched when the respective menu entry is selected "
                        "and its status is 'off' (or, for not monitored entries, when the start submenu "
                        "is selected)." , "" );
  cmdstart_browseb = gtk_button_new_with_label( _("Browse") );
  g_signal_connect_swapped( G_OBJECT(cmdstart_browseb) , "clicked" ,
                            G_CALLBACK(gmu_cb_configwin_modwin_browse) , cmdstart_entry );
  gtk_table_attach( GTK_TABLE(table) , cmdstart_label , 0 , 1 , 2 , 3 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 0 );
  gtk_table_attach( GTK_TABLE(table) , cmdstart_entry , 1 , 2 , 2 , 3 ,
                    GTK_FILL | GTK_EXPAND , GTK_FILL | GTK_EXPAND , 2 , 4 );
  gtk_table_attach( GTK_TABLE(table) , cmdstart_browseb , 2 , 3 , 2 , 3 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 2 );
  cmdstop_label = gtk_label_new( _("Stop CMD:") );
  gtk_misc_set_alignment( GTK_MISC(cmdstop_label) , 0 , 0.5 );
  cmdstop_entry = gtk_entry_new();
  gtk_tooltips_set_tip( tooltips , cmdstop_entry , "The CMDSTOP field is optional; "
                        "the command MUST be specified with absolute path (otherwise it won't work); "
                        "the command specified here will be launched when the respective menu entry is selected "
                        "and its status is 'on' (or, for not monitored entries, when the stop submenu "
                        "is selected)." , "" );
  cmdstop_browseb = gtk_button_new_with_label( _("Browse") );
  g_signal_connect_swapped( G_OBJECT(cmdstop_browseb) , "clicked" ,
                            G_CALLBACK(gmu_cb_configwin_modwin_browse) , cmdstop_entry );
  gtk_table_attach( GTK_TABLE(table) , cmdstop_label , 0 , 1 , 3 , 4 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 0 );
  gtk_table_attach( GTK_TABLE(table) , cmdstop_entry , 1 , 2 , 3 , 4 ,
                    GTK_FILL | GTK_EXPAND , GTK_FILL | GTK_EXPAND , 2 , 4 );
  gtk_table_attach( GTK_TABLE(table) , cmdstop_browseb , 2 , 3 , 3 , 4 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 2 );
  monproc_label = gtk_label_new( _("Mon. File:") );
  gtk_misc_set_alignment( GTK_MISC(monproc_label) , 0 , 0.5 );
  monproc_entry = gtk_entry_new();
  gtk_tooltips_set_tip( tooltips , monproc_entry , "The MONITORED FILE field is optional; "
                        "the file MUST be specified with absolute path (otherwise it won't work); "
                        "the file specified here will be monitored with Gamin (if the 'Mon' flag is set to "
                        "active in the configuration window); when the file exists, the status of the entry "
                        "will be set to 'on' and when the file doesn't exist, the status of the entry will be "
                        "set to 'off'. This will influence which command is launched (start or stop) when the "
                        "entry is selected from the menu." , "" );
  monproc_browseb = gtk_button_new_with_label( _("Browse") );
  g_signal_connect_swapped( G_OBJECT(monproc_browseb) , "clicked" ,
                            G_CALLBACK(gmu_cb_configwin_modwin_browse) , monproc_entry );
  gtk_table_attach( GTK_TABLE(table) , monproc_label , 0 , 1 , 4 , 5 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 0 );
  gtk_table_attach( GTK_TABLE(table) , monproc_entry , 1 , 2 , 4 , 5 ,
                    GTK_FILL | GTK_EXPAND , GTK_FILL | GTK_EXPAND , 2 , 4 );
  gtk_table_attach( GTK_TABLE(table) , monproc_browseb , 2 , 3 , 4 , 5 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 2 );
  iconfile_label = gtk_label_new( _("Icon File:") );
  gtk_misc_set_alignment( GTK_MISC(iconfile_label) , 0 , 0.5 );
  iconfile_entry = gtk_entry_new();
  gtk_tooltips_set_tip( tooltips , iconfile_entry , "The ICON FILE field is optional; "
                        "the file MUST be specified with absolute path (otherwise it won't work); "
                        "the file specified here must be an image, it will be displayed in the menu along "
                        "with the menu entry; if the image size is not 32x32, it will be automatically "
                        "resized (but the original file won't be modified)." , "" );
  iconfile_browseb = gtk_button_new_with_label( _("Browse") );
  g_signal_connect_swapped( G_OBJECT(iconfile_browseb) , "clicked" ,
                            G_CALLBACK(gmu_cb_configwin_modwin_browse) , iconfile_entry );
  gtk_table_attach( GTK_TABLE(table) , iconfile_label , 0 , 1 , 5 , 6 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 0 );
  gtk_table_attach( GTK_TABLE(table) , iconfile_entry , 1 , 2 , 5 , 6 ,
                    GTK_FILL | GTK_EXPAND , GTK_FILL | GTK_EXPAND , 2 , 4 );
  gtk_table_attach( GTK_TABLE(table) , iconfile_browseb , 2 , 3 , 5 , 6 ,
                    GTK_FILL , GTK_FILL | GTK_EXPAND , 2 , 2 );

  gtk_widget_show_all( table );
  gtk_box_pack_start( GTK_BOX(GTK_DIALOG(config_modwindow)->vbox) , table , TRUE , TRUE , 0 );

  /* action-area buttons */
  gtk_button_box_set_spacing( GTK_BUTTON_BOX(GTK_DIALOG(config_modwindow)->action_area) , 0 );
  gtk_dialog_add_buttons( GTK_DIALOG(config_modwindow) ,
                          GTK_STOCK_CANCEL , GTK_RESPONSE_CANCEL ,
                          GTK_STOCK_APPLY , GTK_RESPONSE_APPLY , NULL );

  /* now get configuration window listview pointer from the parent_window pointer */
  ext_lv = g_object_get_data( G_OBJECT(parent_window) , "listview" );
  ext_liststore = gtk_tree_view_get_model( GTK_TREE_VIEW(ext_lv) );

  /* depending on action, fill the entry fields */
  switch ( action )
  {
    case GMU_CONFIGWIN_ADD:
    {
      gtk_window_set_title( GTK_WINDOW(config_modwindow) , _("gMUTOO - add entry") );
      break;
    }

    case GMU_CONFIGWIN_EDIT:
    {
      GtkTreeSelection *ext_listsel;
      gchar *t_name, *t_desc, *t_cmdstart, *t_cmdstop, *t_monproc, *t_iconfile;

      ext_listsel = gtk_tree_view_get_selection( GTK_TREE_VIEW(ext_lv) );

      /* this should -always- be true since it has
         been checked right before calling this function */
      if ( gtk_tree_selection_get_selected( ext_listsel , &ext_liststore , &ext_iter ) )
      {
        /* now ext_iter points to the selected row */
        gtk_tree_model_get( ext_liststore , &ext_iter ,
                            GMU_LVCOLUMN_NAME , &t_name ,
                            GMU_LVCOLUMN_DESC , &t_desc ,
                            GMU_LVCOLUMN_CMDSTART , &t_cmdstart ,
                            GMU_LVCOLUMN_CMDSTOP , &t_cmdstop ,
                            GMU_LVCOLUMN_MONPROC , &t_monproc ,
                            GMU_LVCOLUMN_ICONFILE , &t_iconfile , -1 );

        /* fill the various fields with existing information */
        gtk_entry_set_text( GTK_ENTRY(name_entry) , t_name );
        gtk_entry_set_text( GTK_ENTRY(desc_entry) , t_desc );
        gtk_entry_set_text( GTK_ENTRY(cmdstart_entry) , t_cmdstart );
        gtk_entry_set_text( GTK_ENTRY(cmdstop_entry) , t_cmdstop );
        gtk_entry_set_text( GTK_ENTRY(monproc_entry) , t_monproc );
        gtk_entry_set_text( GTK_ENTRY(iconfile_entry) , t_iconfile );

        g_free( t_name ); g_free( t_desc ); g_free( t_cmdstart );
        g_free( t_cmdstop ); g_free( t_monproc ); g_free( t_iconfile );

        /* set the proper window title */
        gtk_window_set_title( GTK_WINDOW(config_modwindow) , _("gMUTOO - edit entry") );
      }
      else /* however, let's be paranoid and pretend it can fail */
      {
        /* revert to "add entry mode" */
        DEBUGMSG( "no selection found, but edit mode was called... reverting to add mode!\n" );
        gtk_window_set_title( GTK_WINDOW(config_modwindow) , _("gMUTOO - add entry") );
        action = GMU_CONFIGWIN_ADD;
      }

      break;
    }
  }

  while ( done == FALSE )
  {
    gint result = gtk_dialog_run( GTK_DIALOG(config_modwindow) );
    if ( result == GTK_RESPONSE_APPLY )
    {
      gchar *c_name, *c_desc, *c_cmdstart, *c_cmdstop, *c_monproc, *c_iconfile;
      c_name = (gchar*)gtk_entry_get_text( GTK_ENTRY(name_entry) );
      c_desc = (gchar*)gtk_entry_get_text( GTK_ENTRY(desc_entry) );
      c_cmdstart = (gchar*)gtk_entry_get_text( GTK_ENTRY(cmdstart_entry) );
      c_cmdstop = (gchar*)gtk_entry_get_text( GTK_ENTRY(cmdstop_entry) );
      c_monproc = (gchar*)gtk_entry_get_text( GTK_ENTRY(monproc_entry) );
      c_iconfile = (gchar*)gtk_entry_get_text( GTK_ENTRY(iconfile_entry) );

      /* condition 1: name field can't be empty */
      if ( !strcmp( "" , c_name ) )
      {
        gmu_ui_warningbox( GTK_WINDOW(config_modwindow) ,
                           "Entries should have at least\na valid 'name' field!" );
      }
      /* condition 2: if not empty, monproc field must use absolute path */
      else if ( (strcmp( "" , c_monproc )) && (strncmp( "/" , c_monproc , 1 )) )
      {
        gmu_ui_warningbox( GTK_WINDOW(config_modwindow) ,
                           "The monitored file must be\nspecified with an absolute path!" );
      }
      else /* proceed with the requested action */
      {
        switch ( action )
        {
          case GMU_CONFIGWIN_ADD:
          {
            /* for GMU_CONFIGWIN_ADD an extra check is needed;
               avoid that two entries with the same name exists */
            /* we can't modificate directly c_name (got from gtk_entry_get_text) so duplicate it */
            gchar *name_dup = g_strdup( c_name );
            gtk_tree_model_foreach( ext_liststore , gmu_cb_configwin_modwin_checkforequals , name_dup );
            /* if name_dup has been changed to a blank string "",
               it means that an entry with the same name exists */
            if ( !strcmp( name_dup , "" ) )
            {
              gmu_ui_warningbox( GTK_WINDOW(config_modwindow) ,
                                 "A menu entry with the same name\nalready exists!" );
              g_free( name_dup );
              break; /* do not proceed with GMU_CONFIGWIN_EDIT */
            }
            g_free( name_dup );
            /* the iter should point to a new row */
            gtk_list_store_append( GTK_LIST_STORE(ext_liststore) , &ext_iter );
            /* proceed with the commands in GMU_CONFIGWIN_EDIT */
          }

          case GMU_CONFIGWIN_EDIT:
          {
            /* ext_iter has been already properly set elsewhere;
               if this is a GMU_CONFIGWIN_ADD, it points to a new row;
               if this is a GMU_CONFIGWIN_EDIT, it points to the selected row */
            gtk_list_store_set( GTK_LIST_STORE(ext_liststore) , &ext_iter ,
                                GMU_LVCOLUMN_NAME , c_name ,
                                GMU_LVCOLUMN_DESC , c_desc ,
                                GMU_LVCOLUMN_CMDSTART , c_cmdstart ,
                                GMU_LVCOLUMN_CMDSTOP , c_cmdstop ,
                                GMU_LVCOLUMN_IFMON , FALSE ,
                                GMU_LVCOLUMN_MONPROC , c_monproc ,
                                GMU_LVCOLUMN_ICONFILE , c_iconfile ,
                                -1 );
            done = TRUE;
            break;
          }
        }
      }
    }
    else
    {
      done = TRUE;
    }
  }

  gtk_widget_destroy( config_modwindow );
}
